import click

from ixigua import version_info
from ixigua.core import IXiGuaDown


CONTEXT_SETTINGS = dict(
    help_option_names=['-?', '-h', '--help'],
    max_content_width=200,
)

__epilog__ = click.style('''

\b
examples:
    {prog} SINGLE_URL_OR_ID
    {prog} SERIES_URL_OR_ID --playlist
    {prog} SERIES_URL_OR_ID --playlist --rank-prefix
    {prog} SERIES_URL_OR_ID --playlist --playlist-start 10 --playlist-end 20 -O 10-20 --index-prefix 10
    {prog} SERIES_URL_OR_ID --playlist --prefix-pattern '第\d+课'
                              
\x1b[34mcontact: {author} <{author_email}>
''', fg='yellow').format(**version_info)

@click.command(
    name=version_info['prog'],
    help=click.style(version_info['desc'], italic=True, fg='cyan', bold=True),
    context_settings=CONTEXT_SETTINGS,
    no_args_is_help=True,
    epilog=__epilog__,
)
@click.argument('url')
@click.option('-O', '--outdir', help='output directory', default='download', show_default=True)
@click.option('--playlist', help='download playlist', is_flag=True)
@click.option('--playlist-start', help='playlist start index', type=int, default=0)
@click.option('--playlist-end', help='playlist end index', type=int)
@click.option('--rank-prefix', help='add prefix with the rank number', is_flag=True)
@click.option('--index-prefix', help='add prefix with the index number', type=int)
@click.option('--prefix-pattern', help='match prefix pattern from video_abstract, eg. "第\d+课"')
@click.option('--definition', help='the video definition', default='720p')
@click.option('--dryrun', help='dryrun mode', is_flag=True)
@click.version_option(version=version_info['version'], prog_name=version_info['prog'])
def cli(**kwargs):
    ixigua = IXiGuaDown(
        url=kwargs['url'],
        outdir=kwargs['outdir'],
        rank_prefix=kwargs['rank_prefix'],
        index_prefix=kwargs['index_prefix'],
        playlist=kwargs['playlist'],
        playlist_start=kwargs['playlist_start'],
        playlist_end=kwargs['playlist_end'],
    )

    ixigua.download(
        dryrun=kwargs['dryrun'],
        prefix_pattern=kwargs['prefix_pattern'],
        definition=kwargs['definition'],
    )


def main():
    cli()


if __name__ == '__main__':
    main()



import re
import textwrap
from pathlib import Path

from ixigua import utils
from .base import IXiGuaBase


class IXiGuaDown(IXiGuaBase):

    def __init__(self,
                 url: str,
                 outdir='download',
                 rank_prefix=False,
                 index_prefix=None,
                 playlist=False,
                 playlist_start=0,
                 playlist_end=None,
                 playlist_limit=30,
                 ):
        super().__init__(url)

        self.outdir = Path(outdir)
        self.rank_prefix = rank_prefix
        self.index_prefix = index_prefix
        self.playlist = playlist
        self.playlist_start = playlist_start
        self.playlist_end = playlist_end
        self.playlist_limit = playlist_limit

    def get_video_url_new(self):
        """
        anyVideo.gidInformation.packerData.video.videoResource.normal.video_list

        待解决：获取的main_url, 使用base64解码后为乱码，而不是真实的URL
        """

    def get_video_list(self):
        video_list = []
        if self.is_series and self.playlist:
            # 获取合集视频列表
            items = utils.get_series_list(
                pseries_id=self.pseries['id'],
                start=self.playlist_start,
                end=self.playlist_end or self.pseries['seriesInfo']['item_num'],
                limit=self.playlist_limit
            )
            for item in items:
                video_id = item['video_id']
                video_list.append({
                    'item_id': item['item_id'],
                    'vid': video_id,
                    'title': item['title'],
                    'rank': item['rank'],
                })
        else:
            # 获取单视频信息
            video_list.append({
                'item_id': self.video['item_id'],
                'vid': self.video['vid'],
                'title': self.video['title'],
                'rank': self.video['rank'],
            })

        return video_list

    def download(self, dryrun=False, prefix_pattern=None, definition='720p'):
        if not self.ssr_hydrated_data:
            return

        video_list = self.get_video_list()

        start_index = 1
        if self.index_prefix and isinstance(self.index_prefix, int):
            start_index = self.index_prefix

        for n, item in enumerate(video_list, start_index):
            if prefix_pattern:
                xigua = IXiGuaDown(url=item['item_id'])
                if not xigua.ssr_hydrated_data:
                    item['prefix'] = '000. '
                else:
                    video_abstract = xigua.video.get('video_abstract')
                    result = re.findall(prefix_pattern, video_abstract)
                    item['prefix'] = '{}. '.format(result[0]) if result else '111. '
            else:
                item['prefix'] = ''
                if self.index_prefix:
                    item['prefix'] = f'{n}. '
                if self.rank_prefix:
                    item['prefix'] = '{rank}. '.format(**item)

            if dryrun:
                video_url_list = utils.get_video_url_list(video_id=item['vid'])
                item['definitions'] = ', '.join(video_url_list.keys())
                utils.print_item(item)
            else:
                outfile = self.outdir / '{prefix}{title}.mp4'.format(**item)
                utils.download_video(vid=item['vid'], outfile=outfile, definition=definition)



import time
import random

from pathlib import Path
from typing import Union

from webrequests import WebRequest

from .url import get_video_url


def download_video(vid: Union[str, int], outfile: Path, definition='720p'):
    """Download video from vid
    """
    while True:
        try:
            url = get_video_url(vid, definition=definition)
            if outfile.exists():
                resp = WebRequest.get_response(url, method='HEAD')
                content_length = resp.headers['Content-Length']
                if outfile.stat().st_size == int(content_length):
                    print(f'skip download: {outfile}')
                    break
            print(f'>>> downloading: {outfile}')
            WebRequest.download(url, str(outfile))
            break
        except Exception as e:
            time.sleep(random.randint(1, 5))



import textwrap


def print_item(item):
    print('=' * 80)
    print(textwrap.dedent('''
        rank:         {rank}
        title:        {title}
        vid:          {vid}
        item_id:      {item_id}
        definitions:  {definitions}
        prefix:       {prefix}
    ''').format(**item).strip())



import re
import json
from typing import Union

from webrequests import WebRequest
from simple_loggers import SimpleLogger

from ixigua import utils


class IXiGuaBase(object):

    def __init__(self, url: Union[str, int]):
        self.logger = SimpleLogger()
        self.url = utils.wrap_ixigua_url(url)
        self._ssr_hydrated_data = None

    @property
    def ssr_hydrated_data(self):
        """_SSR_HYDRATED_DATA
        """
        if self._ssr_hydrated_data is None:
            response = WebRequest.get_response(self.url, allowed_codes=[200, 404])
            if response.status_code == 404:
                # 404页面报错：视频已下架
                self.logger.debug(f'视频已下架：{self.url}')
                return

            pattern = re.compile(r'_SSR_HYDRATED_DATA=(.*?)</script>', re.DOTALL)
            result = pattern.search(response.text).group(1)
            self._ssr_hydrated_data = json.loads(
                result.replace('undefined', 'null'))
        return self._ssr_hydrated_data

    @property
    def packer_data(self):
        """
        packerData
            - pSeries
                - id
                - seriesInfo
                    - title
                    - item_num
            - video
                - vid
                - title
                - videoResource
                    - normal
                        - video_list
        """
        return self.ssr_hydrated_data['anyVideo']['gidInformation']['packerData']

    @property
    def video(self):
        return self.packer_data['video']

    @property
    def pseries(self):
        return self.packer_data.get('pSeries')

    @property
    def is_series(self):
        return self.pseries is not None



from webrequests import WebRequest


def get_series_list(pseries_id, start=0, end=None, limit=30):
    """
    根据合集ID，获取该合集的视频列表

    https://www.ixigua.com/api/videov2/pseries_more_v2?pSeriesId=6988400843246535199&rank=990&tailCount=30
    rank -- skip
    tailCount -- limit

    Refer: https://www.ixigua.com
    """
    url = 'https://www.ixigua.com/api/videov2/pseries_more_v2'

    headers = {'Referer': 'https://www.ixigua.com'}

    for rank in range(start, end, limit):
        payload = {
            'pSeriesId': pseries_id,
            'rank': rank,
            'tailCount': end - start + 1 if limit > end else limit,
        }

        response = WebRequest.get_response(url, params=payload, headers=headers)
        data = response.json()['data']

        for item in data:
            yield item


import time
import random
from typing import Union
from webrequests import WebRequest



def video_url_base():
    urls = [
        'https://i.snssdk.com',
        'https://vas.snssdk.com',
        'https://vas-lf-x.snssdk.com',
        'https://ib.365yg.com',
    ]
    return random.choice(urls)


def get_video_url_list(video_id: Union[str, int]):
    """
    使用其他接口，根据video_id获取视频列表

    缺点：清晰度最高为720P，无1080P
    """
    # base_url = video_url_base()
    base_url = 'https://i.snssdk.com'

    params = {
        'nobase64': '1',
        't': int(time.time()),
    }

    url = f'{base_url}/video/urls/1/2/3/{video_id}'
    video_list = WebRequest.get_response(url, params=params).json()['data']['video_list']

    data = {}
    for value in video_list.values():
        url = value['main_url']
        definition = value['definition']
        data[definition] = url

    return data

def get_video_url(video_id: Union[str, int], definition='720p'):

    video_urls = get_video_url_list(video_id=video_id)

    video_url = video_urls.get(definition.lower())
    if video_url is None:
        best_definition = sorted(video_urls, key=lambda x: int(x[:-1]))[-1]
        video_url = video_urls[best_definition]

    return video_url

def wrap_ixigua_url(url: Union[str, int]):
    """
    url: 完整的URL或ID

    添加`wid_try=1`，方可获取到渲染后的`_SSR_HYDRATED_DATA`数据
    """
    extra_params = 'wid_try=1'

    if str(url).isdigit():
        return f'https://www.ixigua.com/{url}?{extra_params}'

    if extra_params in url:
        return url

    if '?' in url:
        return url + '&' + extra_params

    return url + '?' + extra_params
